// Main.cpp: Implementierung der Klasse Main.
//
//////////////////////////////////////////////////////////////////////

//#include "Main.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

//Main::Main()
//{

//}

//Main::~Main()
//{

//}

#include "simulation.h"

#include <iostream>
#include <tchar.h>
using std::cout;
using std::cerr;
using std::endl;

#include <Inventor/actions/SoRayPickAction.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/events/SoLocation2Event.h>
#include <Inventor/events/SoMouseButtonEvent.h>
#include <Inventor/nodes/SoCamera.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/projectors/SbPlaneProjector.h>
#include <Inventor/sensors/SoIdleSensor.h>
#include <Inventor/SoPickedPoint.h>
#include <Inventor/Win/SoWin.h>
#include <Inventor/Win/viewers/SoWinExaminerViewer.h>




struct SimData
{
   SoWinExaminerViewer *viewer;
   Simulation *simulation;
};
//typedef struct StrAllData AllData ;


//Idle function
static void idlefunc(void *data, SoSensor* s)
{
   Simulation *simulation=((SimData*)data)->simulation;

   simulation->updateSimulation();
   //cerr << "Simulation updated." << endl;

   s->schedule();
}



//Keyboard callback
static void keyboardfunc(void *data, SoEventCallback *eventCB)
{
   Simulation *simulation=((SimData*)data)->simulation;
   const SoEvent *event = eventCB->getEvent();
   if(SO_KEY_PRESS_EVENT(event, S))
   {
      simulation->pauseSimulation();
      eventCB->setHandled();
   }
}

//Mouse motion callback
static void motionfunc(void *data, SoEventCallback *eventCB)
{
	//cerr << "Mouse movement event." << endl;
	SimData *simData=(SimData*)data;
	SoWinExaminerViewer *viewer=simData->viewer;
	Simulation *simulation=simData->simulation;

	if(!simulation->forceAvailable())
	{
		//cerr << "No force available." << endl;
		return;
	} 

	const SoMouseButtonEvent *mbe=(SoMouseButtonEvent*)eventCB->getEvent();
	SbVec2f pos=mbe->getNormalizedPosition(viewer->getViewportRegion());
	//cerr << "Trying to update force." << endl;
	simulation->updateForce(pos);

}

//Mouse callback
static void mousefunc(void *data, SoEventCallback *eventCB)
{
	SimData *simData=(SimData*)data;
	SoWinExaminerViewer *viewer=simData->viewer;
	Simulation *simulation=simData->simulation;
	const SoMouseButtonEvent *mbe=(SoMouseButtonEvent*)eventCB->getEvent();

	//Handle point grabbing
	if(mbe->getButton() == SoMouseButtonEvent::BUTTON1 && mbe->getState() == SoButtonEvent::DOWN)
	 {
		//cerr << "MButton1 pressed event." << endl;
		//Get viewport point and search for 3D picked point
		SoRayPickAction rp(viewer->getViewportRegion());
		rp.setPoint(mbe->getPosition());
		 rp.apply(viewer->getSceneManager()->getSceneGraph());
		 SoPickedPoint *point = rp.getPickedPoint();
		if(point == NULL)
		 {
			 return;
		 }
		 eventCB->setHandled();

		//Get the first picked node
		SoNode *node=point->getPath()->getTail();
		if(node->getTypeId()!=SoSphere::getClassTypeId())
		{
			 return;
		}

		//It's a sphere
		//Set the projection plane
		cerr << "Force on a SoSphere." << endl;
		SbPlane plane(-viewer->getCamera()->getViewVolume().getProjectionDirection(), point->getPoint());
		SbPlaneProjector *proj=new SbPlaneProjector(plane);
		proj->setViewVolume(viewer->getCamera()->getViewVolume());

		//Grab the corresponding mass point
		simulation->createForce(node->getUserData(), proj);
   }

   //Handle ungrabbing
   if(mbe->getButton() == SoMouseButtonEvent::BUTTON1 && mbe->getState() == SoButtonEvent::UP)
   {
		//cerr << "MButton1 released event." << endl;
		if(simulation->forceAvailable())
		{
			simulation->eraseForce();
		}
		eventCB->setHandled();
   }

}


int _tmain(int argc, _TCHAR* argv[])
{

   HWND window = SoWin::init(argv[0]);
   if (window==NULL) exit(1);

   SoWinExaminerViewer * viewer = new SoWinExaminerViewer(window);

   //Create root
   SoSeparator * root = new SoSeparator;
   root->ref();

   //Initialize scene and add to root
   //Simulation *simulation=new Simulation(argc, argv);
   Simulation *simulation=new Simulation();
   simulation->addSeparator(root);

   //Create application data structure
   SimData simData;
   simData.simulation=simulation;
   simData.viewer=viewer;

   // Create idle sensor and pass pointer to scene
   SoIdleSensor *idle = new SoIdleSensor(idlefunc, &simData);
   idle->schedule();

   // Create event handler for keyboard
   SoEventCallback *keyboardEvent = new SoEventCallback;
   keyboardEvent->addEventCallback(SoKeyboardEvent::getClassTypeId(), keyboardfunc, &simData);
   root->addChild(keyboardEvent);

   // Create event handler for mouse
   SoEventCallback *mouseEvent = new SoEventCallback;
   mouseEvent->addEventCallback(SoMouseButtonEvent::getClassTypeId(), mousefunc, &simData);
   root->addChild(mouseEvent);

   // Create event handler for mouse motion
   SoEventCallback *motionEvent = new SoEventCallback;
   motionEvent->addEventCallback(SoLocation2Event::getClassTypeId(), motionfunc, &simData);
   root->addChild(motionEvent);

   //Start show
   viewer->setSceneGraph(root);
   viewer->show();

   SoWin::show(window);
   SoWin::mainLoop();
   delete viewer;
   root->unref();
   return 0;
}

