// Node.cpp: Implementierung der Klasse Node.
//
//////////////////////////////////////////////////////////////////////
#include <iostream>
using std::iostream;
using std::cout;
using std::cerr;
using std::endl;
#include <tchar.h>

#include "Node.h"
#include "Simulation.h"

#include <Inventor/SbLinear.h>

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

Node::Node(SbVec3f pos, SbVec3f vel)
{
	//default values
	mass = 10.0f;
	damping = 1.0f;
	position = SbVec3f(pos);
	//cerr << "Node: initial position: " << position[0] << position[1] << position[2] << endl;
	velocity = SbVec3f(vel);
	//cerr << "Node: initial velocity: " << velocity[0] << velocity[1] << velocity[2] << endl;
	stiff = false;
	
}

Node::~Node(void)
{

};

void Node::addDamping(void)
{
	force = force - (damping*velocity);
}

void Node::addForce(SbVec3f f)
{
	force = force + f;
}

void Node::addGravity(void)
{
	force = force + SbVec3f(0.0f, -mass*9.81f, 0.0f);
}

void Node::getDerivatives(float *dx, float *dv)
{
	//cerr << "Node: reading dx..." << endl;
	velocity.getValue(dx[0], dx[1], dx[2]);
	//dx = (float*)velocity.getValue();
	//cerr << "Node: ...dx read:" << endl;
	//cerr << dx[0] << ", " << dx[1] << ", " << dx[2] << endl;
	//cerr << "Node: readin dv..." << endl;
	(force/mass).getValue(dv[0], dv[1], dv[2]);
	//dv = (float*)(force/mass).getValue();
	//cerr << "Node: ...dv read:" << endl;
	//cerr << dv[0] << ", " << dv[1] << ", " << dv[2] << endl;
}

void Node::getState(float *x, float *v)
{
   //cerr << "Node: reading position..." << endl;
   position.getValue(x[0], x[1], x[2]);
   //x = (float*)position.getValue();
   //cerr << "Node: ...position read." << endl;
   //cerr << "Node: Reading velocity..." << endl;
   velocity.getValue(v[0], v[1], v[2]);
   //v = (float*)velocity.getValue();
   //cerr << "Node:...velocity read." << endl;
}

void Node::resetForce(void)
{
	force.setValue(0.0f, 0.0f, 0.0f);
}

void Node::setPosition(SbVec3f p)
{
	position = p;
}

void Node::setState(float *x, float *v)
{
   position.setValue(x);
   velocity.setValue(v);
}

void Node::update(void)
{
	if(stiff)
	{
		return;
	}
	velocity = velocity + (force/mass)*Simulation::simStepsize;
	position = position + velocity*Simulation::simStepsize;
}








