// Simulation.h: Schnittstelle fr die Klasse Simulation.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SIMULATION_H__54F1C514_EBD2_445C_8DD0_7FBEEC7578AB__INCLUDED_)
#define AFX_SIMULATION_H__54F1C514_EBD2_445C_8DD0_7FBEEC7578AB__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <vector>
using std::vector;


#include "Node.h"
#include "Spring.h"
#include "Force.h"
#include <Inventor/fields/SoMFInt32.h>
#include <Inventor/lists/SbVec3fList.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/nodes/SoFaceSet.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoTranslation.h>

#include <Inventor/projectors/SbPlaneProjector.h>

#include <Inventor/SbLinear.h>


class Simulation  
{
public:
    static float step;
    static float mass;
    static float stiffness;
    static float damping;

	static float simStepsize;

protected:
	//model
	vector<Node*> nodes;
	vector<Node*> tempNodes;
	vector<Spring*> springs;
	vector<SbPlane*> planes;

	//view
	vector<SoSphere*> spheres;
	vector<SoCylinder*> cylinders;
	
	//transformations
	vector<SoTranslation*> nodeTrans;
	vector<SoTransform*> springTrans;

	//numbers of objects
	int nSprings;
	int nNodes;
	int nMovSprings;
	int nMovNodes;
	int nWalls;

	//roomsize
	float xmin;
	float xmax;
	float ymin;
	float ymax;
	float zmin;
	float zmax;

	float nodeRadius;
	bool pause;
	SbPlaneProjector *projector;
	Force *force;

public:
	Simulation();
	virtual ~Simulation();

	//initialization
	void init();
	void initCG();

	//functions for object creation
	void createNodes();
	void createSprings();
	void createWalls();
	void addSeparator(SoSeparator *root);

	//simulation methods
	void animate();
	void updateSprings();
	void computeForces();
	void detectWallCollisions(float *x, float *v);
	void detectNodeCollisions(float *x, float *v, int nodes, float *x0, Node *tempNodes);
	void pauseSimulation();
	void updateSimulation();
	void getNodeStates(float *x, float *v);
	void setNodeStates(float *x, float *v);
	void getDerivatives(float *dx, float *dv);
	void integrate(float *x, float *v, float t, float *xnew);
	
	void createForce(void *data, SbPlaneProjector *proj);
	void eraseForce();
	void updateForce(SbVec2f &screenPosition);
	bool forceAvailable();

	//Numerical Integration Methods
	void nimLeapFrog();
	void nimRungeKutta4();

	//CG state
	float *x0, *x, *k1, *k2, *k3, *k4;
	float *v0, *v, *l1, *l2, *l3, *l4;
};

#endif // !defined(AFX_SIMULATION_H__54F1C514_EBD2_445C_8DD0_7FBEEC7578AB__INCLUDED_)
